@@ -1,18 +1,18 @@
 package com.googlecode.protobuf.format;
-/* 
+/*
 	Copyright (c) 2009, Orbitz World Wide
 	All rights reserved.
 
-	Redistribution and use in source and binary forms, with or without modification, 
+	Redistribution and use in source and binary forms, with or without modification,
 	are permitted provided that the following conditions are met:
 
-		* Redistributions of source code must retain the above copyright notice, 
+		* Redistributions of source code must retain the above copyright notice,
 		  this list of conditions and the following disclaimer.
-		* Redistributions in binary form must reproduce the above copyright notice, 
-		  this list of conditions and the following disclaimer in the documentation 
+		* Redistributions in binary form must reproduce the above copyright notice,
+		  this list of conditions and the following disclaimer in the documentation
 		  and/or other materials provided with the distribution.
-		* Neither the name of the Orbitz World Wide nor the names of its contributors 
-		  may be used to endorse or promote products derived from this software 
+		* Neither the name of the Orbitz World Wide nor the names of its contributors
+		  may be used to endorse or promote products derived from this software
 		  without specific prior written permission.
 
 	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
@@ -50,6 +50,9 @@ import com.google.protobuf.Descriptors.Descriptor;
 import com.google.protobuf.Descriptors.EnumDescriptor;
 import com.google.protobuf.Descriptors.EnumValueDescriptor;
 import com.google.protobuf.Descriptors.FieldDescriptor;
+import com.googlecode.protobuf.format.util.TextUtils;
+
+import static com.googlecode.protobuf.format.util.TextUtils.*;
 
 /**
  * Provide ascii text parsing and formatting support for proto2 instances. The implementation
@@ -59,19 +62,19 @@ import com.google.protobuf.Descriptors.FieldDescriptor;
  *
  * @author eliran.bivas@gmail.com Eliran Bivas
  * @author aantonov@orbitz.com Alex Antonov
- *         <p/>
+ *         <p>
  *         Based on the original code by:
  * @author wenboz@google.com Wenbo Zhu
  * @author kenton@google.com Kenton Varda
  */
-public class JsonFormat {
+public class JsonFormat extends AbstractCharBasedFormatter {
 
     /**
      * Outputs a textual representation of the Protocol Message supplied into the parameter output.
      * (This representation is the new version of the classic "ProtocolPrinter" output from the
      * original Protocol Buffer system)
      */
-    public static void print(Message message, Appendable output) throws IOException {
+	public void print(final Message message, Appendable output) throws IOException {
         JsonGenerator generator = new JsonGenerator(output);
         generator.print("{");
         print(message, generator);
@@ -81,42 +84,15 @@ public class JsonFormat {
     /**
      * Outputs a textual representation of {@code fields} to {@code output}.
      */
-    public static void print(UnknownFieldSet fields, Appendable output) throws IOException {
+   	public void print(final UnknownFieldSet fields, Appendable output) throws IOException {
         JsonGenerator generator = new JsonGenerator(output);
         generator.print("{");
         printUnknownFields(fields, generator);
         generator.print("}");
     }
 
-    /**
-     * Like {@code print()}, but writes directly to a {@code String} and returns it.
-     */
-    public static String printToString(Message message) {
-        try {
-            StringBuilder text = new StringBuilder();
-            print(message, text);
-            return text.toString();
-        } catch (IOException e) {
-            throw new RuntimeException("Writing to a StringBuilder threw an IOException (should never happen).",
-                                       e);
-        }
-    }
 
-    /**
-     * Like {@code print()}, but writes directly to a {@code String} and returns it.
-     */
-    public static String printToString(UnknownFieldSet fields) {
-        try {
-            StringBuilder text = new StringBuilder();
-            print(fields, text);
-            return text.toString();
-        } catch (IOException e) {
-            throw new RuntimeException("Writing to a StringBuilder threw an IOException (should never happen).",
-                                       e);
-        }
-    }
-
-    protected static void print(Message message, JsonGenerator generator) throws IOException {
+    protected void print(Message message, JsonGenerator generator) throws IOException {
 
         for (Iterator<Map.Entry<FieldDescriptor, Object>> iter = message.getAllFields().entrySet().iterator(); iter.hasNext();) {
             Map.Entry<FieldDescriptor, Object> field = iter.next();
@@ -130,12 +106,12 @@ public class JsonFormat {
         printUnknownFields(message.getUnknownFields(), generator);
     }
 
-    public static void printField(FieldDescriptor field, Object value, JsonGenerator generator) throws IOException {
+    public void printField(FieldDescriptor field, Object value, JsonGenerator generator) throws IOException {
 
         printSingleField(field, value, generator);
     }
 
-    private static void printSingleField(FieldDescriptor field,
+    private void printSingleField(FieldDescriptor field,
                                          Object value,
                                          JsonGenerator generator) throws IOException {
         if (field.isExtension()) {
@@ -189,7 +165,7 @@ public class JsonFormat {
         }
     }
 
-    private static void printFieldValue(FieldDescriptor field, Object value, JsonGenerator generator) throws IOException {
+    private void printFieldValue(FieldDescriptor field, Object value, JsonGenerator generator) throws IOException {
         switch (field.getType()) {
             case INT32:
             case INT64:
@@ -243,7 +219,7 @@ public class JsonFormat {
         }
     }
 
-    protected static void printUnknownFields(UnknownFieldSet unknownFields, JsonGenerator generator) throws IOException {
+    protected void printUnknownFields(UnknownFieldSet unknownFields, JsonGenerator generator) throws IOException {
         boolean firstField = true;
         for (Map.Entry<Integer, UnknownFieldSet.Field> entry : unknownFields.asMap().entrySet()) {
             UnknownFieldSet.Field field = entry.getValue();
@@ -290,29 +266,8 @@ public class JsonFormat {
         }
     }
 
-    /**
-     * Convert an unsigned 32-bit integer to a string.
-     */
-    private static String unsignedToString(int value) {
-        if (value >= 0) {
-            return Integer.toString(value);
-        } else {
-            return Long.toString((value) & 0x00000000FFFFFFFFL);
-        }
-    }
 
-    /**
-     * Convert an unsigned 64-bit integer to a string.
-     */
-    private static String unsignedToString(long value) {
-        if (value >= 0) {
-            return Long.toString(value);
-        } else {
-            // Pull off the most-significant bit so that BigInteger doesn't think
-            // the number is negative, then set it again using setBit().
-            return BigInteger.valueOf(value & 0x7FFFFFFFFFFFFFFFL).setBit(63).toString();
-        }
-    }
+
 
     /**
      * An inner class for writing text to the output stream.
@@ -381,11 +336,11 @@ public class JsonFormat {
 
     /**
      * Represents a stream of tokens parsed from a {@code String}.
-     * <p/>
+     * <p>
      * <p>
      * The Java standard library provides many classes that you might think would be useful for
      * implementing this, but aren't. For example:
-     * <p/>
+     * <p>
      * <ul>
      * <li>{@code java.io.StreamTokenizer}: This almost does what we want -- or, at least, something
      * that would get us close to what we want -- except for one fatal flaw: It automatically
@@ -399,7 +354,7 @@ public class JsonFormat {
      * way to inspect the contents of delimiters, making it impossible to keep track of line and
      * column numbers.
      * </ul>
-     * <p/>
+     * <p>
      * <p>
      * Luckily, Java's regular expression support does manage to be useful to us. (Barely: We need
      * {@code Matcher.usePattern()}, which is new in Java 1.5.) So, we can use that, at least.
@@ -531,6 +486,13 @@ public class JsonFormat {
             }
         }
 
+        /**
+         * Returns {@code true} if the next token is an float, but does not consume it.
+         */
+        public boolean lookingAtFloat() {
+            return lookingAtInteger() && currentToken.contains(".");
+        }
+
         /**
          * Returns {@code true} if the next token is an integer, but does not consume it.
          */
@@ -656,7 +618,7 @@ public class JsonFormat {
                 return Double.NaN;
             }
             try {
-                double result = Double.parseDouble(currentToken);
+                double result = Double.parseDouble(prepareNumberFromString(currentToken));
                 nextToken();
                 return result;
             } catch (NumberFormatException e) {
@@ -681,7 +643,7 @@ public class JsonFormat {
                 return Float.NaN;
             }
             try {
-                float result = Float.parseFloat(currentToken);
+                float result = Float.parseFloat(prepareNumberFromString(currentToken));
                 nextToken();
                 return result;
             } catch (NumberFormatException e) {
@@ -712,7 +674,14 @@ public class JsonFormat {
         public String consumeString() throws ParseException {
           char quote = currentToken.length() > 0 ? currentToken.charAt(0) : '\0';
           if ((quote != '\"') && (quote != '\'')) {
-              throw parseException("Expected string.");
+              try {
+                String result = currentToken.replace(',', '.');
+                Double.parseDouble(result);
+                nextToken();
+                return result;
+              } catch (NumberFormatException e) {
+                throw parseException("Expected string.");
+              }
           }
 
           if ((currentToken.length() < 2)
@@ -732,7 +701,7 @@ public class JsonFormat {
 
         /**
          * If the next token is a string, consume it, unescape it as a
-         * {@link com.googlecode.protobuf.format.ByteString}, and return it. Otherwise, throw a
+         * {@link ByteString}, and return it. Otherwise, throw a
          * {@link ParseException}.
          */
         public ByteString consumeByteString() throws ParseException {
@@ -804,61 +773,12 @@ public class JsonFormat {
         }
     }
 
-    /**
-     * Parse a text-format message from {@code input} and merge the contents into {@code builder}.
-     */
-    public static void merge(Readable input, Message.Builder builder) throws IOException {
-        merge(input, ExtensionRegistry.getEmptyRegistry(), builder);
-    }
-
-    /**
-     * Parse a text-format message from {@code input} and merge the contents into {@code builder}.
-     */
-    public static void merge(CharSequence input, Message.Builder builder) throws ParseException {
-        merge(input, ExtensionRegistry.getEmptyRegistry(), builder);
-    }
 
     /**
      * Parse a text-format message from {@code input} and merge the contents into {@code builder}.
      * Extensions will be recognized if they are registered in {@code extensionRegistry}.
      */
-    public static void merge(Readable input,
-                             ExtensionRegistry extensionRegistry,
-                             Message.Builder builder) throws IOException {
-        // Read the entire input to a String then parse that.
-
-        // If StreamTokenizer were not quite so crippled, or if there were a kind
-        // of Reader that could read in chunks that match some particular regex,
-        // or if we wanted to write a custom Reader to tokenize our stream, then
-        // we would not have to read to one big String. Alas, none of these is
-        // the case. Oh well.
-
-        merge(toStringBuilder(input), extensionRegistry, builder);
-    }
-
-    private static final int BUFFER_SIZE = 4096;
-
-    // TODO(chrisn): See if working around java.io.Reader#read(CharBuffer)
-    // overhead is worthwhile
-    protected static StringBuilder toStringBuilder(Readable input) throws IOException {
-        StringBuilder text = new StringBuilder();
-        CharBuffer buffer = CharBuffer.allocate(BUFFER_SIZE);
-        while (true) {
-            int n = input.read(buffer);
-            if (n == -1) {
-                break;
-            }
-            buffer.flip();
-            text.append(buffer, 0, n);
-        }
-        return text;
-    }
-
-    /**
-     * Parse a text-format message from {@code input} and merge the contents into {@code builder}.
-     * Extensions will be recognized if they are registered in {@code extensionRegistry}.
-     */
-    public static void merge(CharSequence input,
+    public void merge(CharSequence input,
                              ExtensionRegistry extensionRegistry,
                              Message.Builder builder) throws ParseException {
         Tokenizer tokenizer = new Tokenizer(input);
@@ -875,15 +795,12 @@ public class JsonFormat {
         }
     }
 
-    private static final Pattern DIGITS = Pattern.compile(
-          "[0-9]",
-          Pattern.CASE_INSENSITIVE);
 
     /**
      * Parse a single field from {@code tokenizer} and merge it into {@code builder}. If a ',' is
      * detected after the field ends, the next field will be parsed automatically
      */
-    protected static void mergeField(Tokenizer tokenizer,
+    protected void mergeField(Tokenizer tokenizer,
                                    ExtensionRegistry extensionRegistry,
                                    Message.Builder builder) throws ParseException {
         FieldDescriptor field;
@@ -915,11 +832,11 @@ public class JsonFormat {
 
         // Last try to lookup by field-index if 'name' is numeric,
         // which indicates a possible unknown field
-        if (field == null && DIGITS.matcher(name).matches()) {
+        if (field == null && TextUtils.isDigits(name)) {
             field = type.findFieldByNumber(Integer.parseInt(name));
             unknown = true;
         }
-        
+
         // Finally, look for extensions
         extension = extensionRegistry.findExtensionByName(name);
         if (extension != null) {
@@ -959,7 +876,7 @@ public class JsonFormat {
         }
     }
 
-    private static void handleMissingField(Tokenizer tokenizer,
+    private void handleMissingField(Tokenizer tokenizer,
                                            ExtensionRegistry extensionRegistry,
                                            Message.Builder builder) throws ParseException {
         tokenizer.tryConsume(":");
@@ -982,6 +899,8 @@ public class JsonFormat {
             // Primitive value
             if ("null".equals(tokenizer.currentToken())) {
                 tokenizer.consume("null");
+            } else if (tokenizer.lookingAtFloat()) {
+                tokenizer.consumeFloat();
             } else if (tokenizer.lookingAtInteger()) {
                 tokenizer.consumeInt64();
             } else if (tokenizer.lookingAtBoolean()) {
@@ -992,7 +911,7 @@ public class JsonFormat {
         }
     }
 
-    private static void handleValue(Tokenizer tokenizer,
+    private void handleValue(Tokenizer tokenizer,
                                     ExtensionRegistry extensionRegistry,
                                     Message.Builder builder,
                                     FieldDescriptor field,
@@ -1014,7 +933,7 @@ public class JsonFormat {
         }
     }
 
-    private static Object handlePrimitive(Tokenizer tokenizer, FieldDescriptor field) throws ParseException {
+    private Object handlePrimitive(Tokenizer tokenizer, FieldDescriptor field) throws ParseException {
         Object value = null;
         if ("null".equals(tokenizer.currentToken())) {
             tokenizer.consume("null");
@@ -1096,7 +1015,7 @@ public class JsonFormat {
         return value;
     }
 
-    private static Object handleObject(Tokenizer tokenizer,
+    private Object handleObject(Tokenizer tokenizer,
                                        ExtensionRegistry extensionRegistry,
                                        Message.Builder builder,
                                        FieldDescriptor field,
@@ -1198,7 +1117,7 @@ public class JsonFormat {
         }
         return builder.toString();
     }
-	
+
 	static String unicodeEscaped(char ch) {
 		if (ch < 0x10) {
 			return "\\u000" + Integer.toHexString(ch);
@@ -1326,7 +1245,7 @@ public class JsonFormat {
     /**
      * Implements JSON string escaping as specified <a href="http://www.ietf.org/rfc/rfc4627.txt">here</a>.
      * <ul>
-     *  <li>The following characters are escaped by prefixing them with a '\' : \b,\f,\n,\r,\t,\,"</li> 
+     *  <li>The following characters are escaped by prefixing them with a '\' : \b,\f,\n,\r,\t,\,"</li>
      *  <li>Other control characters in the range 0x0000-0x001F are escaped using the \\uXXXX notation</li>
      *  <li>UTF-16 surrogate pairs are encoded using the \\uXXXX\\uXXXX notation</li>
      *  <li>any other character is printed as-is</li>
@@ -1453,34 +1372,6 @@ public class JsonFormat {
       return builder.toString();
     }
 
-    /**
-     * Is this an octal digit?
-     */
-    private static boolean isOctal(char c) {
-        return ('0' <= c) && (c <= '7');
-    }
-
-    /**
-     * Is this a hex digit?
-     */
-    private static boolean isHex(char c) {
-        return (('0' <= c) && (c <= '9')) || (('a' <= c) && (c <= 'f'))
-               || (('A' <= c) && (c <= 'F'));
-    }
-
-    /**
-     * Interpret a character as a digit (in any base up to 36) and return the numeric value. This is
-     * like {@code Character.digit()} but we don't accept non-ASCII digits.
-     */
-    private static int digitValue(char c) {
-        if (('0' <= c) && (c <= '9')) {
-            return c - '0';
-        } else if (('a' <= c) && (c <= 'z')) {
-            return c - 'a' + 10;
-        } else {
-            return c - 'A' + 10;
-        }
-    }
 
     /**
      * Parse a 32-bit signed integer from the text. Unlike the Java standard {@code
@@ -1540,7 +1431,7 @@ public class JsonFormat {
             radix = 8;
         }
 
-        String numberText = text.substring(pos);
+        String numberText = prepareNumberFromString(text.substring(pos));
 
         long result = 0;
         if (numberText.length() < 16) {
@@ -1604,4 +1495,19 @@ public class JsonFormat {
 
         return result;
     }
+
+    /**
+     * Accept "" empty string
+     * and number in quote "6" as number
+     */
+    private static String prepareNumberFromString(String numberText) {
+
+        numberText = numberText.replaceAll("\"", "");
+
+        if (numberText.equals("")) {
+            numberText = "0"; // default value
+        }
+
+        return numberText;
+    }
 }
