@@ -45,6 +45,7 @@ import com.google.protobuf.Message;
 import com.google.protobuf.UnknownFieldSet;
 import com.google.protobuf.Descriptors.EnumValueDescriptor;
 import com.google.protobuf.Descriptors.FieldDescriptor;
+import static com.googlecode.protobuf.format.util.TextUtils.*;
 
 /**
  * Provide ascii text parsing and formatting support for proto2 instances. The implementation
@@ -54,19 +55,19 @@ import com.google.protobuf.Descriptors.FieldDescriptor;
  *
  * @author eliran.bivas@gmail.com Eliran Bivas
  * @author aantonov@orbitz.com Alex Antonov
- *         <p/>
+ *         <p>
  *         Based on the original code by:
  * @author wenboz@google.com Wenbo Zhu
  * @author kenton@google.com Kenton Varda
  */
-public final class XmlFormat {
+public final class XmlFormat extends AbstractCharBasedFormatter {
 
     /**
      * Outputs a textual representation of the Protocol Message supplied into the parameter output.
      * (This representation is the new version of the classic "ProtocolPrinter" output from the
      * original Protocol Buffer system)
      */
-    public static void print(Message message, Appendable output) throws IOException {
+    public void print(final Message message, Appendable output) throws IOException {
         XmlGenerator generator = new XmlGenerator(output);
         final String messageName = message.getDescriptorForType().getName();
         generator.print("<");
@@ -81,42 +82,15 @@ public final class XmlFormat {
     /**
      * Outputs a textual representation of {@code fields} to {@code output}.
      */
-    public static void print(UnknownFieldSet fields, Appendable output) throws IOException {
+    public void print(final UnknownFieldSet fields, Appendable output) throws IOException {
         XmlGenerator generator = new XmlGenerator(output);
         generator.print("<message>");
         printUnknownFields(fields, generator);
         generator.print("</message>");
     }
 
-    /**
-     * Like {@code print()}, but writes directly to a {@code String} and returns it.
-     */
-    public static String printToString(Message message) {
-        try {
-            StringBuilder text = new StringBuilder();
-            print(message, text);
-            return text.toString();
-        } catch (IOException e) {
-            throw new RuntimeException("Writing to a StringBuilder threw an IOException (should never happen).",
-                                       e);
-        }
-    }
 
-    /**
-     * Like {@code print()}, but writes directly to a {@code String} and returns it.
-     */
-    public static String printToString(UnknownFieldSet fields) {
-        try {
-            StringBuilder text = new StringBuilder();
-            print(fields, text);
-            return text.toString();
-        } catch (IOException e) {
-            throw new RuntimeException("Writing to a StringBuilder threw an IOException (should never happen).",
-                                       e);
-        }
-    }
-
-    private static void print(Message message, XmlGenerator generator) throws IOException {
+    private void print(Message message, XmlGenerator generator) throws IOException {
 
         for (Map.Entry<FieldDescriptor, Object> field : message.getAllFields().entrySet()) {
             printField(field.getKey(), field.getValue(), generator);
@@ -124,7 +98,7 @@ public final class XmlFormat {
         printUnknownFields(message.getUnknownFields(), generator);
     }
 
-    public static void printField(FieldDescriptor field, Object value, XmlGenerator generator) throws IOException {
+    public void printField(FieldDescriptor field, Object value, XmlGenerator generator) throws IOException {
 
         if (field.isRepeated()) {
             // Repeated field. Print each element.
@@ -136,7 +110,7 @@ public final class XmlFormat {
         }
     }
 
-    private static void printSingleField(FieldDescriptor field, Object value, XmlGenerator generator) throws IOException {
+    private void printSingleField(FieldDescriptor field, Object value, XmlGenerator generator) throws IOException {
         if (field.isExtension()) {
             generator.print("<extension type=\"");
             // We special-case MessageSet elements for compatibility with
@@ -178,7 +152,7 @@ public final class XmlFormat {
 
     }
 
-    private static void printFieldValue(FieldDescriptor field, Object value, XmlGenerator generator) throws IOException {
+    private void printFieldValue(FieldDescriptor field, Object value, XmlGenerator generator) throws IOException {
         switch (field.getType()) {
             case INT32:
             case INT64:
@@ -224,7 +198,7 @@ public final class XmlFormat {
         }
     }
 
-    private static void printUnknownFields(UnknownFieldSet unknownFields, XmlGenerator generator) throws IOException {
+    private void printUnknownFields(UnknownFieldSet unknownFields, XmlGenerator generator) throws IOException {
         for (Map.Entry<Integer, UnknownFieldSet.Field> entry : unknownFields.asMap().entrySet()) {
             UnknownFieldSet.Field field = entry.getValue();
 
@@ -251,7 +225,7 @@ public final class XmlFormat {
         }
     }
 
-    private static void printUnknownField(CharSequence fieldKey,
+    private void printUnknownField(CharSequence fieldKey,
                                           CharSequence fieldValue,
                                           XmlGenerator generator) throws IOException {
         generator.print("<unknown-field index=\"");
@@ -261,35 +235,12 @@ public final class XmlFormat {
         generator.print("</unknown-field>");
     }
 
-    /**
-     * Convert an unsigned 32-bit integer to a string.
-     */
-    private static String unsignedToString(int value) {
-        if (value >= 0) {
-            return Integer.toString(value);
-        } else {
-            return Long.toString((value) & 0x00000000FFFFFFFFL);
-        }
-    }
-
-    /**
-     * Convert an unsigned 64-bit integer to a string.
-     */
-    private static String unsignedToString(long value) {
-        if (value >= 0) {
-            return Long.toString(value);
-        } else {
-            // Pull off the most-significant bit so that BigInteger doesn't
-            // think
-            // the number is negative, then set it again using setBit().
-            return BigInteger.valueOf(value & 0x7FFFFFFFFFFFFFFFL).setBit(63).toString();
-        }
-    }
+   
 
     /**
      * An inner class for writing text to the output stream.
      */
-    static private final class XmlGenerator {
+    private static final class XmlGenerator {
 
         Appendable output;
 
@@ -320,11 +271,11 @@ public final class XmlFormat {
 
     /**
      * Represents a stream of tokens parsed from a {@code String}.
-     * <p/>
+     * <p>
      * <p>
      * The Java standard library provides many classes that you might think would be useful for
      * implementing this, but aren't. For example:
-     * <p/>
+     * <p>
      * <ul>
      * <li>{@code java.io.StreamTokenizer}: This almost does what we want -- or, at least, something
      * that would get us close to what we want -- except for one fatal flaw: It automatically
@@ -338,7 +289,7 @@ public final class XmlFormat {
      * way to inspect the contents of delimiters, making it impossible to keep track of line and
      * column numbers.
      * </ul>
-     * <p/>
+     * <p>
      * <p>
      * Luckily, Java's regular expression support does manage to be useful to us. (Barely: We need
      * {@code Matcher.usePattern()}, which is new in Java 1.5.) So, we can use that, at least.
@@ -695,74 +646,12 @@ public final class XmlFormat {
         }
     }
 
-    /**
-     * Thrown when parsing an invalid text format message.
-     */
-    public static class ParseException extends IOException {
-
-    	private static final long serialVersionUID = 1L;
-
-		public ParseException(String message) {
-            super(message);
-        }
-    }
-
-    /**
-     * Parse a text-format message from {@code input} and merge the contents into {@code builder}.
-     */
-    public static void merge(Readable input, Message.Builder builder) throws ParseException,
-                                                                     IOException {
-        XmlFormat.merge(input, ExtensionRegistry.getEmptyRegistry(), builder);
-    }
-
-    /**
-     * Parse a text-format message from {@code input} and merge the contents into {@code builder}.
-     */
-    public static void merge(CharSequence input, Message.Builder builder) throws ParseException {
-        merge(input, ExtensionRegistry.getEmptyRegistry(), builder);
-    }
-
+    
     /**
      * Parse a text-format message from {@code input} and merge the contents into {@code builder}.
      * Extensions will be recognized if they are registered in {@code extensionRegistry}.
      */
-    public static void merge(Readable input,
-                             ExtensionRegistry extensionRegistry,
-                             Message.Builder builder) throws ParseException, IOException {
-        // Read the entire input to a String then parse that.
-
-        // If StreamTokenizer were not quite so crippled, or if there were a kind
-        // of Reader that could read in chunks that match some particular regex,
-        // or if we wanted to write a custom Reader to tokenize our stream, then
-        // we would not have to read to one big String. Alas, none of these is
-        // the case. Oh well.
-
-        merge(toStringBuilder(input), extensionRegistry, builder);
-    }
-
-    private static final int BUFFER_SIZE = 4096;
-
-    // TODO(chrisn): See if working around java.io.Reader#read(CharBuffer)
-    // overhead is worthwhile
-    private static StringBuilder toStringBuilder(Readable input) throws IOException {
-        StringBuilder text = new StringBuilder();
-        CharBuffer buffer = CharBuffer.allocate(BUFFER_SIZE);
-        while (true) {
-            int n = input.read(buffer);
-            if (n == -1) {
-                break;
-            }
-            buffer.flip();
-            text.append(buffer, 0, n);
-        }
-        return text;
-    }
-
-    /**
-     * Parse a text-format message from {@code input} and merge the contents into {@code builder}.
-     * Extensions will be recognized if they are registered in {@code extensionRegistry}.
-     */
-    public static void merge(CharSequence input,
+    public void merge(CharSequence input,
                              ExtensionRegistry extensionRegistry,
                              Message.Builder builder) throws ParseException {
         Tokenizer tokenizer = new Tokenizer(input);
@@ -777,21 +666,21 @@ public final class XmlFormat {
         consumeClosingElement(tokenizer);
     }
 
-    private static String consumeOpeningElement(Tokenizer tokenizer) throws ParseException {
+    private String consumeOpeningElement(Tokenizer tokenizer) throws ParseException {
         tokenizer.consume("<");
         String openingElement = tokenizer.consumeIdentifier();
         tokenizer.consume(">");
         return openingElement;
     }
 
-    private static void consumeClosingElement(Tokenizer tokenizer) throws ParseException {
+    private void consumeClosingElement(Tokenizer tokenizer) throws ParseException {
         tokenizer.tryConsume("</");
         //tokenizer.consume("/");
         tokenizer.nextToken();
         tokenizer.consume(">");
     }
 
-    private static String consumeExtensionIdentifier(Tokenizer tokenizer) throws ParseException {
+    private String consumeExtensionIdentifier(Tokenizer tokenizer) throws ParseException {
         tokenizer.consume("type");
         tokenizer.consume("=");
         return tokenizer.consumeIdentifier();
@@ -801,7 +690,7 @@ public final class XmlFormat {
      * Parse a single field from {@code tokenizer} and merge it into {@code builder}. If a ',' is
      * detected after the field ends, the next field will be parsed automatically
      */
-    private static void mergeField(Tokenizer tokenizer,
+    private void mergeField(Tokenizer tokenizer,
                                    ExtensionRegistry extensionRegistry,
                                    Message.Builder builder) throws ParseException {
         FieldDescriptor field;
@@ -875,7 +764,7 @@ public final class XmlFormat {
         consumeClosingElement(tokenizer);
     }
 
-    private static Object handleValue(Tokenizer tokenizer,
+    private Object handleValue(Tokenizer tokenizer,
                                     ExtensionRegistry extensionRegistry,
                                     Message.Builder builder,
                                     FieldDescriptor field,
@@ -891,7 +780,7 @@ public final class XmlFormat {
         return value;
     }
 
-    private static Object handlePrimitive(Tokenizer tokenizer, FieldDescriptor field) throws ParseException {
+    private Object handlePrimitive(Tokenizer tokenizer, FieldDescriptor field) throws ParseException {
         Object value = null;
         switch (field.getType()) {
             case INT32:
@@ -969,7 +858,7 @@ public final class XmlFormat {
         return value;
     }
 
-    private static Object handleObject(Tokenizer tokenizer,
+    private Object handleObject(Tokenizer tokenizer,
                                        ExtensionRegistry extensionRegistry,
                                        Message.Builder builder,
                                        FieldDescriptor field,
@@ -1182,34 +1071,6 @@ public final class XmlFormat {
         return unescapeBytes(input).toStringUtf8();
     }
 
-    /**
-     * Is this an octal digit?
-     */
-    private static boolean isOctal(char c) {
-        return ('0' <= c) && (c <= '7');
-    }
-
-    /**
-     * Is this a hex digit?
-     */
-    private static boolean isHex(char c) {
-        return (('0' <= c) && (c <= '9')) || (('a' <= c) && (c <= 'f'))
-               || (('A' <= c) && (c <= 'F'));
-    }
-
-    /**
-     * Interpret a character as a digit (in any base up to 36) and return the numeric value. This is
-     * like {@code Character.digit()} but we don't accept non-ASCII digits.
-     */
-    private static int digitValue(char c) {
-        if (('0' <= c) && (c <= '9')) {
-            return c - '0';
-        } else if (('a' <= c) && (c <= 'z')) {
-            return c - 'a' + 10;
-        } else {
-            return c - 'A' + 10;
-        }
-    }
 
     /**
      * Parse a 32-bit signed integer from the text. Unlike the Java standard {@code
